<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class JccOrderMapping extends ObjectModel
{
    const SANDBOX = 0;
    
    const LIVE = 1;

    public $id;

    public $idOrder;

    public $referenceNo;

    public $ps_id_order;
    
    public $type;

    public static $definition = array(
        'table' => 'nt_jcc_order_mapping',
        'primary' => 'id_nt_jcc_order_mapping',
        'fields' => array(
            'idOrder' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'referenceNo' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 64
            ),
            'ps_id_order' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'type' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true)
        ),
    );

    public static function getOrderDetails($idOrder)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(static::$definition['table']);
        $sql->where('ps_id_order = "'.pSQL($idOrder) .'"');
        return (Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql));
    }


    public static function getTotalPartialRefundAmount($idOrder)
    {
        $sql = new DbQuery();
        $sql->select('SUM(product_quantity) product_quantity');
        $sql->select('SUM(amount_tax_excl) amount_tax_excl');
        $sql->select('SUM(amount_tax_incl) amount_tax_incl');
        $sql->from('order_slip_detail', 'osd');
        $sql->leftJoin('order_slip', 'os', 'os.`id_order_slip` = osd.`id_order_slip`');
        $sql->where('os.`id_order` = '.(int)$idOrder);
        return (Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql));
    }
}
