<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class JccpaymentConfirmationModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        if (Tools::getValue('OrderID') == false) {
            return false;
        }
        
        $responseOrderId = explode('-', Tools::getValue('OrderID'));

        if ($this->isValidOrder() === true) {
            $payment_status = Configuration::get('PS_OS_PAYMENT');
            $message = null;
            
            $cart_id = $responseOrderId[1];
            $cart = new Cart((int) $cart_id);
            $customer = new Customer((int) $cart->id_customer);
            
            if (!Validate::isLoadedObject($customer)) {
                Tools::redirect('index.php?controller=order&step=1');
                return;
            }
            
            $module_name = $this->module->displayName;
            $currency_id = (int) Context::getContext()->currency->id;
    
            $extraParams = array(
                'transaction_id' => trim(Tools::getValue('ReferenceNo'))
            );

            $this->module->validateOrder(
                $cart_id,
                $payment_status,
                $cart->getOrderTotal(),
                $module_name,
                $message,
                $extraParams,
                $currency_id,
                false,
                $customer->secure_key
            );

            $order_id = Order::getOrderByCartId((int) $cart->id);
            
            $ntOrderLog = array(
                'order_id' => $order_id,
                'id_cart' => $cart->id,
                'transaction_id' => trim(Tools::getValue('ReferenceNo'))
            );
            $this->addSuccessLog("Order Success", $ntOrderLog);
            if ($order_id) {
                $module_id = $this->module->id;
                $this->mapOrderDetails(Tools::getAllValues(), $order_id);
                $redirectUrl = 'index.php?controller=order-confirmation&id_cart=' . $cart_id;
                $redirectUrl .= '&id_module=' . $module_id .'&id_order=' . $order_id.'&key=' . $customer->secure_key;
                Tools::redirect($redirectUrl);
            }
        } else {
            $payment_status = Configuration::get('PS_OS_ERROR');

            $paymentErrorMessage = 'An error occurred while processing payment';
            if (!empty(Tools::getValue('ReasonCodeDesc'))) {
                $paymentErrorMessage = Tools::getValue('ReasonCodeDesc');
            }
            if (Tools::getValue('ResponseCode') == 2) {
                $paymentErrorMessage = 'Transaction was declined, Please try again.';
            }

            $message = $this->module->l($paymentErrorMessage);
            $this->errors[] = $message;
            $this->addErrorLog($message, Tools::getAllValues());
            $this->redirectWithNotifications('index.php?controller=order&step=1');
            // Tools::redirect('index.php?controller=order&step=1');
        }
    }
    
    protected function isValidOrder()
    {
        if (Tools::getValue('ResponseCode') == 1) {
            return true;
        }
        return false;
    }

    protected function mapOrderDetails($details, $idOrder)
    {
        try {
            $objJccOrderMapping = new JccOrderMapping();
            $objJccOrderMapping->idOrder = $details['OrderID'];
            $objJccOrderMapping->referenceNo = $details['ReferenceNo'];
            $objJccOrderMapping->ps_id_order = $idOrder;
            $paymentMode = Configuration::get('JCCPAYMENT_LIVE_MODE');
            if ($paymentMode == 1) {
                $objJccOrderMapping->type = JccOrderMapping::LIVE;
            } else {
                $objJccOrderMapping->type = JccOrderMapping::SANDBOX;
            }
            $objJccOrderMapping->save();
        } catch (Exception $e) {
        }
    }

    public function addSuccessLog($msg, $orderDetails)
    {
        $today = getdate();
        $logFile = _PS_MODULE_DIR_.'jccpayment/logs/success/'.$today['mon'].'_'.$today['mday'];
        $logFile .= '_'.$today['year'].'.log';
        if ($errorLog = fopen($logFile, 'a+')) {
            $now = new DateTime();
            $txt = 'Transaction Success  ['.$now->format('Y-m-d H:i:s').'] : ';
            $txt .= "\n";
            if ($msg) {
                $txt .= "MSG ". $msg;
                $txt .= "\n";
            }

            if ($msg) {
                $txt .= "order details ". print_r($orderDetails, true);
                $txt .= "\n";
            }
            
            $txt .= "\n ------------------------------------------------------------------------ ";
            fwrite($errorLog, $txt."\n");
        }
    }

    public function addErrorLog($msg, $orderDetails, $amount = false)
    {
        $today = getdate();
        $logFile = _PS_MODULE_DIR_.$this->name.'/logs/errors/'.$today['mon'].'_'.$today['mday'];
        $logFile .= '_'.$today['year'].'.log';
        if ($errorLog = fopen($logFile, 'a+')) {
            $now = new DateTime();
            $txt = 'Jcc Transaction  ['.$now->format('Y-m-d H:i:s').'] : ';
            $txt .= "\n";
            if ($msg) {
                $txt .= "MSG ". $msg;
                $txt .= "\n";
            }
            if ($amount) {
                $txt .= "amount ". $amount;
                $txt .= "\n";
            }
            if ($msg) {
                $txt .= "order details ". print_r($orderDetails, true);
                $txt .= "\n";
            }
            
            $txt .= "\n ------------------------------------------------------------------------ ";
            fwrite($errorLog, $txt."\n");
        }
    }
}
