<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class JccpaymentRedirectModuleFrontController extends ModuleFrontController
{
    /**
     * Do whatever you have to before redirecting the customer on the website of your payment processor.
     */
    public function postProcess()
    {
        /*
         * Oops, an error occured.
         */
        if (Tools::getValue('action') == 'error') {
            return $this->displayError('An error occurred while trying to redirect the customer');
        } else {
            $cart = $this->context->cart;

            $version = '1.0.0';
            $paymentMode = Configuration::get('JCCPAYMENT_LIVE_MODE');
            $acquirerID = Configuration::get('JCCPAYMENT_ACQUIRER_ID');
            $merchantID = ($paymentMode == 1)
                            ? Configuration::get('JCCPAYMENT_MERCHANT_ID')
                            : Configuration::get('JCCPAYMENT_SANDBOX_MERCHANT_ID');
            $password = ($paymentMode == 1)
                        ? Configuration::get('JCCPAYMENT_PASSWORD')
                        : Configuration::get('JCCPAYMENT_SANDBOX_PASSWORD');
            $orderAmount = $cart->getOrderTotal(true, Cart::BOTH);
            $ntOrderAmount = (int) ($orderAmount * pow(10, _PS_PRICE_DISPLAY_PRECISION_));
            $orderAmount = str_pad($orderAmount, 13, "0", STR_PAD_LEFT);
            $ntOrderAmount = str_pad($ntOrderAmount, 12, "0", STR_PAD_LEFT);
            // $formattedPurchaseAmt = Tools::substr($orderAmount, 0, 10).Tools::substr($orderAmount, 11);
            $formattedPurchaseAmt = $ntOrderAmount;
            // dump($formattedPurchaseAmt);
            // die;
            $currencyExp = 2;
            $merchantOrderIdPrefix = Configuration::get('JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX');
            $orderID = !empty($merchantOrderIdPrefix) ? $merchantOrderIdPrefix.'-'.$cart->id : uniqid().'-'.$cart->id;
            $currencyObject = new Currency($cart->id_currency);
            $currencyCode = $currencyObject->iso_code_num;
            $toEncrypt = $password.$merchantID.$acquirerID.$orderID.$formattedPurchaseAmt.$currencyCode;
            $sha1Signature = sha1($toEncrypt);
            $base64Sha1Signature = base64_encode(pack("H*", $sha1Signature));
            $signatureMethod = "SHA1";
            
            $customerData = new Customer($cart->id_customer);
            $customerEmail = $customerData->email;
            
            $sendBillingInfo = false;
            $sendBillingData = [];
            if (Configuration::get('JCCPAYMENT_SEND_BILLING_INFO')) {
                $sendBillingInfo = true;
                $billingAddress = new Address($cart->id_address_invoice);
                $sendBillingData['firstname'] = $this->escapeSpecial(
                    $billingAddress->firstname
                );
                $sendBillingData['lastname'] = $this->escapeSpecial(
                    $billingAddress->lastname
                );
                $sendBillingData['address1'] = $this->escapeSpecial(
                    $billingAddress->address1
                );
                $sendBillingData['address2'] = $this->escapeSpecial(
                    $billingAddress->address2
                );
                $sendBillingData['postcode'] = $this->escapeSpecial(
                    $billingAddress->postcode
                );
                $sendBillingData['city'] = $this->escapeSpecial(
                    $billingAddress->city
                );
                $sendBillingData['phone'] = $this->escapeSpecial(
                    $billingAddress->phone
                );
                $sendBillingData['email'] = $customerEmail;
            }
            
            $sendShippingInfo = false;
            $sendShippingData = [];
            if (Configuration::get('JCCPAYMENT_SEND_SHIPPING_INFO')) {
                $sendShippingInfo = true;
                $deliveryAddress = new Address($cart->id_address_delivery);
                $sendShippingData['firstname'] = $this->escapeSpecial(
                    $deliveryAddress->firstname
                );
                $sendShippingData['lastname'] = $this->escapeSpecial(
                    $deliveryAddress->lastname
                );
                $sendShippingData['address1'] = $this->escapeSpecial(
                    $deliveryAddress->address1
                );
                $sendShippingData['address2'] = $this->escapeSpecial(
                    $deliveryAddress->address2
                );
                $sendShippingData['postcode'] = $this->escapeSpecial(
                    $deliveryAddress->postcode
                );
                $sendShippingData['city'] = $this->escapeSpecial(
                    $deliveryAddress->city
                );
                $sendShippingData['phone'] = $this->escapeSpecial(
                    $deliveryAddress->phone
                );
                $sendShippingData['email'] = $customerEmail;
            }
            
            $sendGeneralInfo = false;
            $sendGeneralData = [];
            if (Configuration::get('JCCPAYMENT_SEND_SHIPPING_INFO')) {
                $sendGeneralInfo = true;
                $sendGeneralData['email'] = $customerEmail;
                if ($cart->id_address_invoice == $cart->id_address_delivery) {
                    $sendGeneralData['addressMatch'] = 'Y';
                } else {
                    $sendGeneralData['addressMatch'] = 'N';
                }
            }
            
            
            if ($paymentMode == 1) {
                $formActionUrl = 'https://jccpg.jccsecure.com/EcomPayment/RedirectAuthLink';
            } else {
                $formActionUrl = 'https://tjccpg.jccsecure.com/EcomPayment/RedirectAuthLink';
            }

            $merchantCallbackUrl = '';
            if (Configuration::get('JCCPAYMENT_MERCHANT_CALLBACK_SWITCH')) {
                $merchantCallbackUrl = Configuration::get('JCCPAYMENT_MERCHANT_CALLBACK_URL');
            } else {
                // $merchantCallbackUrl = $this->context->link->getModuleLink(
                //     $this->module->name,
                //     'confirmation',
                //     array(),
                //     true
                // );
            }
            $this->context->smarty->assign(array(
                'version' => $version,
                'merchantID' => $merchantID,
                'acquirerID' => $acquirerID,
                'responseURL' => $this->context->link->getModuleLink(
                    $this->module->name,
                    'confirmation',
                    array(),
                    true
                ),
                'merchantCallbackUrl' => $merchantCallbackUrl,
                'formattedPurchaseAmt' => $formattedPurchaseAmt,
                'currencyCode' => $currencyCode,
                'currencyExp' => $currencyExp,
                'captureFlag' => Configuration::get('JCCPAYMENT_CAPTURE_FLAG'),
                'password' => $password,
                'base64Sha1Signature' => $base64Sha1Signature,
                'orderID' => $orderID,
                'signatureMethod' => $signatureMethod,
                'formActionUrl' => $formActionUrl,
                'sendBillingInfo' => $sendBillingInfo,
                'sendBillingData' => $sendBillingData,
                'sendShippingInfo' => $sendShippingInfo,
                'sendShippingData' => $sendShippingData,
                'sendGeneralInfo' => $sendGeneralInfo,
                'sendGeneralData' => $sendGeneralData
            ));

            return $this->setTemplate('module:jccpayment/views/templates/front/redirect.tpl');
        }
    }

    protected function displayError($message, $description = false)
    {
        $this->context->smarty->assign('path', '
			<a href="' .
            $this->context->link->getPageLink(
                'order',
                null,
                null,
                'step=3'
            ) .
            '">' . $this->module->l('Payment') . '</a>
            <span class="navigation-pipe">&gt;</span>' .
        $this->module->l('Error'));

        /*
         * Set error message and description for the template.
         */
        array_push($this->errors, $this->module->l($message), $description);

        return $this->setTemplate('module:jccpayment/views/templates/front/error.tpl');
    }

    public function escapeSpecial($string)
    {
        return htmlspecialchars($string, ENT_QUOTES);
    }
}
