<?php
/**
* 2007-2021 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2021 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__).'/classes/requiredclasses.php';

class Jccpayment extends PaymentModule
{
    protected $html = '';

    public function __construct()
    {
        $this->name = 'jccpayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.1';
        $this->author = 'JCC Payment Systems';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();
        
        $this->limited_currencies = array('EUR', 'RUB', 'CHF', 'GBP', 'USD');

        $this->displayName = $this->l('JCC Payment Gateway for PrestaShop');
        $this->description = $this->l('Accepts payments using JCC Payment Gateway');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        Configuration::updateValue('JCCPAYMENT_LIVE_MODE', false);
        Configuration::updateValue('JCCPAYMENT_ACQUIRER_ID', null);
        Configuration::updateValue('JCCPAYMENT_SANDBOX_MERCHANT_ID', null);
        Configuration::updateValue('JCCPAYMENT_SANDBOX_PASSWORD', null);
        Configuration::updateValue('JCCPAYMENT_MERCHANT_ID', null);
        Configuration::updateValue('JCCPAYMENT_PASSWORD', null);
        Configuration::updateValue('JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX', null);
        Configuration::updateValue('JCCPAYMENT_CAPTURE_FLAG', 'A');
        Configuration::updateValue('JCCPAYMENT_SEND_BILLING_INFO', false);
        Configuration::updateValue('JCCPAYMENT_SEND_SHIPPING_INFO', false);
        Configuration::updateValue('JCCPAYMENT_SEND_GENERAL_INFO', false);
        Configuration::updateValue(
            'JCCPAYMENT_SANDBOX_FINANCIAL_WSDL',
            'https://tjccpg.jccsecure.com/PgWebService/services/FinancialService?wsdl'
        );
        Configuration::updateValue(
            'JCCPAYMENT_FINANCIAL_WSDL',
            'https://jccpg.jccsecure.com/PgWebService/services/FinancialService?wsdl'
        );
        Configuration::updateValue('JCCPAYMENT_MERCHANT_CALLBACK_SWITCH', false);
        Configuration::updateValue(
            'JCCPAYMENT_MERCHANT_CALLBACK_URL',
            $this->context->link->getModuleLink(
                $this->name,
                'confirmation'
            )
        );

        $objPaymentDb = new JccPaymentDb();
        $objPaymentDb->createTable();
        return parent::install() &&
            $this->registerPsHooks();
    }

    public function registerPsHooks()
    {
        $hooks = array(
            'paymentOptions',
            'displayAdminOrderTop',
            'displayAdminOrder',
            'actionOrderSlipAdd',
            'actionOrderStatusPostUpdate'
        );
        foreach ($hooks as $hook) {
            $this->registerHook($hook);
        }
        return true;
    }

    public function uninstall()
    {
        Configuration::deleteByName('JCCPAYMENT_LIVE_MODE');
        Configuration::deleteByName('JCCPAYMENT_ACQUIRER_ID');
        Configuration::deleteByName('JCCPAYMENT_SANDBOX_MERCHANT_ID');
        Configuration::deleteByName('JCCPAYMENT_SANDBOX_PASSWORD');
        Configuration::deleteByName('JCCPAYMENT_MERCHANT_ID');
        Configuration::deleteByName('JCCPAYMENT_PASSWORD');
        Configuration::deleteByName('JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX');
        Configuration::deleteByName('JCCPAYMENT_CAPTURE_FLAG');
        Configuration::deleteByName('JCCPAYMENT_SEND_BILLING_INFO');
        Configuration::deleteByName('JCCPAYMENT_SEND_SHIPPING_INFO');
        Configuration::deleteByName('JCCPAYMENT_SEND_GENERAL_INFO');
        Configuration::deleteByName('JCCPAYMENT_SANDBOX_FINANCIAL_WSDL');
        Configuration::deleteByName('JCCPAYMENT_FINANCIAL_WSDL');
        Configuration::deleteByName('JCCPAYMENT_MERCHANT_CALLBACK_URL');
        Configuration::deleteByName('JCCPAYMENT_MERCHANT_CALLBACK_SWITCH');

        $objPaymentDb = new JccPaymentDb();
        $objPaymentDb->deleteTables();
        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitJccpaymentModule')) == true) {
            $this->postProcess();
        }

        return $this->html.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitJccpaymentModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $descriptionContent = $this->l('(Contact JCC to whitelist the IPs of your server before enabling');
        $descriptionContent .= $this->l(' this option – May cause delay if IPs are not whitelisted)');
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'JCCPAYMENT_LIVE_MODE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_ACQUIRER_ID',
                        'label' => $this->l('Acquirer Id'),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_SANDBOX_MERCHANT_ID',
                        'label' => $this->l('Sandbox Merchant Id'),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_SANDBOX_PASSWORD',
                        'label' => $this->l('Sandbox Password'),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_MERCHANT_ID',
                        'label' => $this->l('Production Merchant Id'),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_PASSWORD',
                        'label' => $this->l('Production Password'),
                    ),
                    array(
                        'col' => 2,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX',
                        'label' => $this->l('Merchant Order Id Prefix'),
                        'maxlength' => 10
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Capture Type'),
                        'name' => 'JCCPAYMENT_CAPTURE_FLAG',
                        'options' => array(
                         'query' => array(
                              array(
                                'id_option' => 'A',
                                'name' => 'Automatic'
                              ),
                              array(
                                'id_option' => 'M',
                                'name' => 'Manual'
                              ),
                          ),
                          'id' => 'id_option',
                          'name' => 'name'
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Send Billing Info'),
                        'name' => 'JCCPAYMENT_SEND_BILLING_INFO',
                        'desc' => $this->l('Allow the plugin to send billing info to the Issuing Bank
                                            in order to perform a real-time risk scoring 
                                            of the transaction according to EMV 3DS
                                            (Billing Address 1
                                            , Billing Address 2
                                            , Billing City
                                            , Billing Country
                                            , Billing First Name
                                            , Billing Last Name
                                            , Billing Postal Code
                                            , Billing Email
                                            , Billing Mobile Phone
                                            , Billing State)'),
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Send Shipping Info'),
                        'name' => 'JCCPAYMENT_SEND_SHIPPING_INFO',
                        'desc' => $this->l('Allow the plugin to send shipping info to the Issuing Bank
                                            in order to perform a real-time risk scoring 
                                            of the transaction according to EMV 3DS
                                            (Shipping Address 1
                                            , Shipping Address 2
                                            , Shipping State
                                            , Shipping City
                                            , Shipping Country
                                            , Shipping Postal Code
                                            , Shipping Method Indicator
                                            , Shipping First Name
                                            , Shipping Last Name)'),
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                     array(
                        'type' => 'switch',
                        'label' => $this->l('Send General Info'),
                        'name' => 'JCCPAYMENT_SEND_GENERAL_INFO',
                        'desc' => $this->l('Allow the plugin to send general info to the Issuing Bank
                                            in order to perform a real-time risk scoring 
                                            of the transaction according to EMV 3DS
                                            (Shipping And Billing Addresses Match
                                            , Order Description
                                            , Delivery Time Frame
                                            , Delivery Email)'),
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_SANDBOX_FINANCIAL_WSDL',
                        'label' => $this->l('Sandbox Financial Service WSDL'),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_FINANCIAL_WSDL',
                        'label' => $this->l('Financial Service WSDL'),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable Merchant Callback Url'),
                        'name' => 'JCCPAYMENT_MERCHANT_CALLBACK_SWITCH',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'text',
                        'name' => 'JCCPAYMENT_MERCHANT_CALLBACK_URL',
                        'label' => $this->l('Merchant Callback URL'),
                        'desc' => $descriptionContent,
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        if (!empty(Configuration::get('JCCPAYMENT_SANDBOX_PASSWORD'))) {
            $sandboxPassword = '*******';
        } else {
            $sandboxPassword = Configuration::get('JCCPAYMENT_SANDBOX_PASSWORD', '');
        }

        if (!empty(Configuration::get('JCCPAYMENT_PASSWORD'))) {
            $password = '*******';
        } else {
            $password = Configuration::get('JCCPAYMENT_PASSWORD', '');
        }


        return array(
            'JCCPAYMENT_LIVE_MODE' => Configuration::get('JCCPAYMENT_LIVE_MODE', false),
            'JCCPAYMENT_ACQUIRER_ID' => Configuration::get('JCCPAYMENT_ACQUIRER_ID', ''),
            'JCCPAYMENT_SANDBOX_MERCHANT_ID' => Configuration::get('JCCPAYMENT_SANDBOX_MERCHANT_ID', ''),
            'JCCPAYMENT_SANDBOX_PASSWORD' => $sandboxPassword,
            'JCCPAYMENT_MERCHANT_ID' => Configuration::get('JCCPAYMENT_MERCHANT_ID', ''),
            'JCCPAYMENT_PASSWORD' => $password,
            'JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX' => Configuration::get('JCCPAYMENT_MERCHANT_ORDER_ID_PREFIX', ''),
            'JCCPAYMENT_CAPTURE_FLAG' => Configuration::get('JCCPAYMENT_CAPTURE_FLAG', 'A'),
            'JCCPAYMENT_SEND_BILLING_INFO' => Configuration::get('JCCPAYMENT_SEND_BILLING_INFO', false),
            'JCCPAYMENT_SEND_SHIPPING_INFO' => Configuration::get('JCCPAYMENT_SEND_SHIPPING_INFO', false),
            'JCCPAYMENT_SEND_GENERAL_INFO' => Configuration::get('JCCPAYMENT_SEND_GENERAL_INFO', false),
            'JCCPAYMENT_SANDBOX_FINANCIAL_WSDL' => Configuration::get('JCCPAYMENT_SANDBOX_FINANCIAL_WSDL', ''),
            'JCCPAYMENT_FINANCIAL_WSDL' => Configuration::get('JCCPAYMENT_FINANCIAL_WSDL', ''),
            'JCCPAYMENT_MERCHANT_CALLBACK_SWITCH' => Configuration::get('JCCPAYMENT_MERCHANT_CALLBACK_SWITCH', ''),
            'JCCPAYMENT_MERCHANT_CALLBACK_URL' => Configuration::get('JCCPAYMENT_MERCHANT_CALLBACK_URL', ''),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            if ($key == 'JCCPAYMENT_PASSWORD' || $key == 'JCCPAYMENT_SANDBOX_PASSWORD') {
                if (Tools::getValue($key) != '*******') {
                    Configuration::updateValue($key, Tools::getValue($key));
                }
            } else {
                Configuration::updateValue($key, Tools::getValue($key));
            }
        }

        $this->html = $this->displayConfirmation(
            $this->trans('Settings have been updated successfully.', array(), 'Admin.Notifications.Success')
        );
        return $this->html;
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('JCC Gateway'))
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', array(), true));

        return [
            $option
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookDisplayAdminOrderTop($params)
    {
        $return = $this->getPartialRefund($params);

        return $return;
    }

    public function hookDisplayAdminOrder($params)
    {
        // Since Ps 1.7.7 this hook is displayed at bottom of a page and we should use a hook DisplayAdminOrderTop
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return false;
        }

        $return = $this->getPartialRefund($params);

        return $return;
    }

    protected function getPartialRefund($params)
    {
        $orderDetails = JccOrderMapping::getOrderDetails($params['id_order']);
        if (!$orderDetails) {
            return '';
        }

        $this->context->smarty->assign('chb_jcc_refund', $this->l('Refund on JCC'));
        return $this->context->smarty->fetch(_PS_MODULE_DIR_ . $this->name . '/views/templates/hook/partialRefund.tpl');
    }

    public function hookActionOrderSlipAdd($params)
    {
        if (Tools::isSubmit('doPartialRefundJccPayment')) {
            $orderDetails = JccOrderMapping::getOrderDetails($params['order']->id);
            if (!$orderDetails) {
                return false;
            }

            $refundResponse = $this->partialRefund($params);

            $refundAmountResult = $refundResponse->RefundAmountResult; // boolean
            $sError = $refundResponse->sError;

            if ($refundResponse) {
                if ($refundAmountResult == true && $sError == 'Success') {
                    $this->addSuccessLog("Refunded success", $orderDetails);
                } else {
                    $msg = "Order  failed to be refunded due to " . $sError;
                    $this->addErrorLog($msg, $orderDetails);
                }
            } else {
                $this->addErrorLog("Order failed to be refunded", $refundResponse);
            }
        }
    }


    // public function hookActionOrderStatusPostUpdate(&$params)
    // {
    //     $idOrder = (int)$params['id_order'];
    //     $orderDetails = JccOrderMapping::getOrderDetails($idOrder);
    //     if (!$orderDetails) {
    //         return false;
    //     }

    //     $osCanceled = (int)Configuration::get('PS_OS_CANCELED');

    //     $osRefunded = (int)Configuration::get('PS_OS_REFUND');

    //     if ($params['newOrderStatus']->id == $osCanceled
    //         || $params['newOrderStatus']->id == $osRefunded
    //     ) {
    //         $refundResponse = $this->refund($idOrder);

    //         $refundAmountResult = $jccFinancialServiceResponse->RefundAmountResult; // boolean
    //         $sError = $jccFinancialServiceResponse->sError;


    //         $refundAmountResult = $refundResponse->RefundAmountResult; // boolean
    //         $sError = $refundResponse->sError;

    //         if ($refundResponse) {
    //             if ($refundAmountResult == true && $sError == 'Success') {
    //                 $this->addSuccessLog("Refunded success", $orderDetails);
    //             } else {
    //                 $msg = "Order  failed to be refunded due to " . $sError;
    //                 $this->addErrorLog($msg, $orderDetails);
    //             }
    //         } else {
    //             $this->addErrorLog("Order failed to be refunded", $orderDetails);
    //         }
    //     }
    // }

    public function refund($idOrder)
    {
        $orderDetails = JccOrderMapping::getOrderDetails($idOrder);
        if ($orderDetails) {
            $amount = 0;


            $objOrder = new Order($idOrder);
            $amount = $objOrder->getTotalPaid();
            $refundedAmount = JccOrderMapping::getTotalPartialRefundAmount($idOrder);
            if ($refundedAmount && isset($refundedAmount['amount_tax_incl'])) {
                $refundAmount = (float)$refundedAmount['amount_tax_incl'];
                $refundAmount = Tools::ps_round($refundAmount, 2);
                $amount = $amount - $refundAmount;
            }

            $orderKey = $orderDetails['idOrder'];
            if ($amount > 0 && !empty($orderKey)) {
                try {
                    return $this->doRefund($orderKey, $orderDetails['type
                    '], $amount);
                } catch (Exception $e) {
                    $this->addErrorLog($e->getMessage(), $amount, $orderDetails);
                }
        
                return false;
            }
        }
    }

    public function partialRefund($params)
    {
        $orderDetails = JccOrderMapping::getOrderDetails($params['order']->id);
        if ($orderDetails) {
            $amount = 0;

            foreach ($params['productList'] as $product) {
                $amount += Tools::ps_round($product['amount'], 2);
            }
    
            if (Tools::getValue('partialRefundShippingCost')) {
                $amount += Tools::getValue('partialRefundShippingCost');
            }
    
            // For prestashop version > 1.7.7
            if ($refundData = Tools::getValue('cancel_product')) {
                $amount += (float)(str_replace(',', '.', $refundData['shipping_amount']));
            }

            $amount = Tools::ps_round($amount, 2);
            $orderKey = $orderDetails['idOrder'];

            // if ($amount > 0 && !empty($orderKey)) {

            try {
                return $this->doRefund($orderKey, $orderDetails['type
                '], $amount);
            } catch (Exception $e) {
                $this->addErrorLog($e->getMessage(), $amount, $orderDetails);
            }
            // }
            return false;
        }
    }

    public function doRefund($idOrder, $type, $amount = null)
    {
        ini_set('soap.wsdl_cache_enabled', 0);
        ini_set('soap.wsdl_cache_ttl', 900);
        ini_set('default_socket_timeout', 15);

        $options = array(
            'cache_wsdl' => WSDL_CACHE_NONE,
            'trace' => true,
            'exceptions' => 1,
            'soap_version' => SOAP_1_1,
        );

        // $liveMode = Configuration::get('JCCPAYMENT_LIVE_MODE');
        if ($type == JccOrderMapping::SANDBOX) {
            $financialServiceWsdl = Configuration::get('JCCPAYMENT_SANDBOX_FINANCIAL_WSDL');
            $password = Configuration::get('JCCPAYMENT_SANDBOX_PASSWORD');
            $idMerchant = Configuration::get('JCCPAYMENT_SANDBOX_MERCHANT_ID');
        } else {
            $financialServiceWsdl = Configuration::get('JCCPAYMENT_FINANCIAL_WSDL');
            $password = Configuration::get('JCCPAYMENT_PASSWORD');
            $idMerchant = Configuration::get('JCCPAYMENT_MERCHANT_ID');
        }

        $financialServiceSoapClient = new SoapClient($financialServiceWsdl, $options);

        $AuthHeader = new stdClass();
        $AuthHeader->MerchantID = $idMerchant;
        $AuthHeader->AcquirerID = Configuration::get('JCCPAYMENT_ACQUIRER_ID');
        $AuthHeader->Password = $password;

        $context = stream_context_create(array('http' => array('header' => 'Accept: application/xml')));
        $xml =  Tools::file_get_contents($financialServiceWsdl, false, $context);
        $xml = simplexml_load_string($xml);
        $target_namespace = $xml->attributes()->targetNamespace;

        $header = new SoapHeader($target_namespace, 'AuthHeader', $AuthHeader, false);
        //set the Headers of Soap Client.
        $financialServiceSoapClient->__setSoapHeaders($header);

        $Culture = '';
        $sError = 'Success';

        $refundAmountClass = new stdClass();
        $refundAmountClass->OrderID = $idOrder;
        $refundAmountClass->Amount = $amount;
        $refundAmountClass->Culture = $Culture;
        $refundAmountClass->sError = $sError;

        $financialServiceResponse = $financialServiceSoapClient->RefundAmount($refundAmountClass);

        return $financialServiceResponse;
    }

    public function addSuccessLog($msg, $orderDetails)
    {
        $today = getdate();
        $logFile = _PS_MODULE_DIR_.$this->name.'/logs/success/'.$today['mon'].'_'.$today['mday'];
        $logFile .= '_'.$today['year'].'.log';
        if ($errorLog = fopen($logFile, 'a+')) {
            $now = new DateTime();
            $txt = 'Jcc Refund  ['.$now->format('Y-m-d H:i:s').'] : ';
            $txt .= "\n";
            if ($msg) {
                $txt .= "MSG ". $msg;
                $txt .= "\n";
            }

            if ($msg) {
                $txt .= "order details ". print_r($orderDetails, true);
                $txt .= "\n";
            }
            
            $txt .= "\n ------------------------------------------------------------------------ ";
            fwrite($errorLog, $txt."\n");
        }
    }

    public function addErrorLog($msg, $orderDetails, $amount = false)
    {
        $today = getdate();
        $logFile = _PS_MODULE_DIR_.$this->name.'/logs/errors/'.$today['mon'].'_'.$today['mday'];
        $logFile .= '_'.$today['year'].'.log';
        if ($errorLog = fopen($logFile, 'a+')) {
            $now = new DateTime();
            $txt = 'Jcc Refund  ['.$now->format('Y-m-d H:i:s').'] : ';
            $txt .= "\n";
            if ($msg) {
                $txt .= "MSG ". $msg;
                $txt .= "\n";
            }
            if ($amount) {
                $txt .= "amount ". $amount;
                $txt .= "\n";
            }
            if ($msg) {
                $txt .= "order details ". print_r($orderDetails, true);
                $txt .= "\n";
            }
            
            $txt .= "\n ------------------------------------------------------------------------ ";
            fwrite($errorLog, $txt."\n");
        }
    }
}
